# 	$Id: hmm_select.pl,v 1.4 2009/11/25 01:44:42 genis Exp $	

use strict; use warnings; use sigtrap;
use FAlite;
use Getopt::Std;
use vars qw($opt_p $opt_n $opt_i);

getopts('p:n:i:');

my $PREFIX = "";
my $PROTNUM;
my $HMM_PREFIX ="KOG";

$PREFIX = $opt_n if $opt_n;
$PROTNUM = $opt_p if $opt_p;
$HMM_PREFIX = $opt_i if $opt_i;
die "

# This script takes a protein multifasta file and run it against a hmm 
# profile using hmmsearch. 

hmm_select.pl <hmm_directory> <protein_fasta_file> <cutoff_file>
options:
   -n <file_name>    # extended output

" unless (@ARGV == 3);


my $hmm_directory = $ARGV[0];
my $prot_file = $ARGV[1];
my $cutoff_file = $ARGV[2];

# Security check to not overwrite the results
if (!(-e "$prot_file" )) {
    print "File does not exist: $prot_file \n";
    exit(1);
}

my %score;
my %score_cutoff;
my %best_pred;

open(CUTOFF_FILE, "< $cutoff_file") or die;

while (<CUTOFF_FILE>){
    if (/($HMM_PREFIX[0-9]+)\s(.*)/){
	$score_cutoff{$1}=$2;
	$score{$1}=0;
    }
}

close(CUTOFF_FILE);

open(EXTENDED_FILE, "> extended.tbl") or die;

# Opening the protein fasta files
open(PROT_FILE, $prot_file) or die;
my $PROT = new FAlite(\*PROT_FILE);

system ("rm -f hmm_select.aln");

# Main loop of execution
while (my $prot_entry = $PROT->nextEntry) {

    # Opening temporary fasta files
    open(TMP_PROT, "> /tmp/prot$$.fa") or die;
    print_fasta($prot_entry,\*TMP_PROT);
    my ($kog_id) = $prot_entry->def =~ /($HMM_PREFIX\d+)/;
    close(TMP_PROT);
    
    print STDERR "$kog_id \n";
    #print "$kog_id \n";
    
    # running hmmsearch
    system ("hmmsearch $hmm_directory/$kog_id.hmm /tmp/prot$$.fa > /tmp/hmm$$.output");

    open(TMP_HMM, "< /tmp/hmm$$.output");
    while (<TMP_HMM>){
	if (/($kog_id.[0-9]+)_.*AA\s+([-0-9\.]+)\s+([-0-9e\.]+)\s+([-0-9\.]+)/){
	    # copy  the score to the extended result
	    print EXTENDED_FILE "$1 $2 $score_cutoff{$kog_id}\n";
	    if ($2 > $score_cutoff{$kog_id} && $2 > $score{$kog_id}){
		$score{$kog_id} = $2;
		$best_pred{$kog_id} = $1;
		#print "$1 $2 $score_cutoff{$kog_id}\n";
	    }
	}
    }
    close(TMP_HMM);

    # copy the output in the final file:
    system ("cat /tmp/hmm$$.output >> hmm_select.aln ");

}

close (PROT_FILE);


open(PROT_FILE, $prot_file) or die;
my $PROT2 = new FAlite(\*PROT_FILE);

# Printing the proteins

while (my $prot_entry = $PROT2->nextEntry) {
    my ($kog_pred_id) = $prot_entry->def =~ /($HMM_PREFIX\d+.[0-9]+)/;
    my ($kog_id) = $prot_entry->def =~ /($HMM_PREFIX\d+)/;
    if ($best_pred{$kog_id} && $best_pred{$kog_id} eq $kog_pred_id){
	print_fasta_STDOUT($prot_entry);
    }

}



# Printing all the final results

#my $key;
#foreach $key (keys %score){
#    if ($score{$key} > 0){
#	print "$key\n";
#    }
#}




sub print_fasta {

    my ($entry, $file) = @_;
    my ($id, $seq) = split (/\n/,$entry);
    my ($i, $e) = (1, length($seq));
    print $file "$id\n";
    while ($i<=$e) {
	print  $file  substr($seq,$i-1,60)."\n"; 
	$i += 60;
    };
}

sub print_fasta_STDOUT {

    my ($entry) = @_;
    my ($id, $seq) = split (/\n/,$entry);
    my ($i, $e) = (1, length($seq));
    print  "$id\n";
    while ($i<=$e) {
	print  substr($seq,$i-1,60)."\n"; 
	$i += 60;
    };
}
