######################################################################
#                            local_map                               #
######################################################################
#
#     cegma pipeline code
#
######################################################################
#      
# 	$Id: local_map.pl,v 1.4 2009/11/25 01:33:59 genis Exp $	

use strict; use warnings; use sigtrap;
use FAlite;
use Getopt::Std;
use vars qw($opt_p $opt_h $opt_n $opt_g $opt_d $opt_f $opt_i);

##########################################################################
##                                                                      ##
##                     VARIABLES  AND DEFINITIONS                       ##
##                                                                      ##
##########################################################################
 
my $CEGMA  = defined($ENV{'CEGMA'}) ? $ENV{'CEGMA'} : '.';
my $TMP    = defined($ENV{'CEGMATMP'}) ? $ENV{'CEGMATMP'} : '/tmp';

getopts('p:n:h:g:d:fi:');

my $PREFIX = "";
my $HMMDIR;
my $PROTOPT;
my $WISE_GFF;
my $GENEID="$CEGMA/data/generic.param";
my $HMM_PREFIX ="KOG";


$PREFIX = $opt_n if $opt_n;
$HMMDIR = $opt_h if $opt_h;
$PROTOPT = $opt_p if $opt_p;
$WISE_GFF = $opt_g if $opt_g;
$GENEID = $opt_d if $opt_d;
$HMM_PREFIX = $opt_i if $opt_i;

die "

# This script takes a protien multifasta file and a genomic multifasta file
# and runs genewise and geneid+genewise and evaluate the results.

# we considered the sequence of the proteins and the genomic to be 
# in the same order.

local_map.pl <genomic_fasta_file>
options:
   -n <prefix of the output files >
   -p <prot_fasta_file>
   -h <directory with the hmm profiles>
   -g <genewise_gff>    # This option skips genewise  
   -d <geneid_parameter>
   -f

" unless (@ARGV == 1 && ( $opt_h || $opt_p ));

# Bash variable:

my $file1 = $ARGV[0];


# Security check to not overwrite the results
if (!(-e "$file1" )) {
    print "File does not exist: $file1 \n";
    exit(1);
}


if (-e "$PREFIX.genewise" ) {
    print "File exist: $PREFIX.genewise \n";
    exit(1);
}

# Opening the fasta files
open(GENOMIC_FILE, $file1) or die;
my $GENOMIC = new FAlite(\*GENOMIC_FILE);

my $PROT;

# Proteins option
if ($PROTOPT) {
    open(PROT_FILE, $PROTOPT) or die;
    $PROT = new FAlite(\*PROT_FILE);
    open(TMP_PROT, "> $TMP/prot$$.fa") or die;
}

# Opening temporary fasta file
open(TMP_GENOMIC, "> $TMP/genomic$$.fa") or die;


# Main loop of execution
while (my $gen_entry = $GENOMIC->nextEntry) {

    # Opening temporary fasta files
    open(TMP_GENOMIC, "> $TMP/genomic$$.fa") or die;
    print_fasta($gen_entry,\*TMP_GENOMIC);
    my ($kog_id) = $gen_entry->def =~ /($HMM_PREFIX[0-9]+)/;        
    my ($kog_id_sub) = $gen_entry->def =~ /($HMM_PREFIX[0-9.]+)/;        
    
    close(TMP_GENOMIC);
    
    print STDERR "$kog_id_sub \n";
    print "$kog_id_sub \n";
    
    # if we are running with a single protein
    if ($PROTOPT) {
	open(TMP_PROT, "> $TMP/prot$$.fa") or die;
	my ($prot_entry) = $PROT->nextEntry;
	print_fasta($prot_entry,\*TMP_PROT);  
	close(TMP_PROT);
    }      

    if (!($WISE_GFF)) {
	# running genewise
	if ($PROTOPT) {
	    system ("genewise -splice_gtag -quiet  -gff -pretty -alb $TMP/prot$$.fa $TMP/genomic$$.fa \\
               > $TMP/genewise$$");
	}else{
	    system ("genewise -splice_gtag -quiet -gff -pretty -alb  -hmmer $HMMDIR/$kog_id.hmm $TMP/genomic$$.fa \\
               > $TMP/genewise$$");
	}
	system ("cat $TMP/genewise$$ >> $PREFIX.genewise");
	# filtering the result
	system ("$CEGMA/bin/parsewise $TMP/genewise$$ > $TMP/genewise$$.gff ");
	
	system ("cat $TMP/genewise$$.gff >> $PREFIX.genewise.gff");
	system ("echo \"#\$\" >> $PREFIX.genewise.gff");
    } else {
	# genewise file provided
	system ("grep -w $kog_id_sub $WISE_GFF > $TMP/genewise$$.gff");
	
    }


    # Obtanining the boundaries of the genewise alignment
    open(TMP_GENEWISE, "< $TMP/genewise$$.gff") or die;

    my $initial_genewise_coord=0;
    my $final_genewise_coord=0;
    my $counter=0;

    while (<TMP_GENEWISE>){
    	my @gff = split;
	if ($counter == 0){
	    $initial_genewise_coord=$gff[3];
	}
	$counter++;
	$final_genewise_coord=$gff[4];
    }

    close(TMP_GENEWISE);    
   
    # running geneid

    open(TMP_GENOMIC, "> $TMP/genomic_mask$$.fa") or die;
    print_fasta_masked($gen_entry,\*TMP_GENOMIC, $initial_genewise_coord-1000, $final_genewise_coord+1000);
    close(TMP_GENOMIC);
 
    if ($opt_f){
	system ("geneid -P $GENEID -S $TMP/genewise$$.gff \\
                 -WGF $TMP/genomic_mask$$.fa >> $PREFIX.geneid.gff");
        system ("geneid -AP $GENEID -S $TMP/genewise$$.gff \\
                 -WF $TMP/genomic_mask$$.fa >> $PREFIX.geneid");
    } else {
	system ("geneid -P $GENEID -S $TMP/genewise$$.gff \\
                 -WG $TMP/genomic_mask$$.fa >> $PREFIX.geneid.gff");
        system ("geneid -AP $GENEID -S $TMP/genewise$$.gff \\
                 -W $TMP/genomic_mask$$.fa >> $PREFIX.geneid");
    }
    
    system ("echo \"#\$\" >> $PREFIX.geneid.gff");

    system ("echo \"#\$\" >> $PREFIX.geneid");

}


system ("awk '{if (substr(\$1,1,1)==\">\") bol=1;if (\$0==\"\") bol=0;if (bol) print; }' $PREFIX.geneid > $PREFIX.geneid.fa");


close (PROT_FILE);
close (GENOMIC_FILE);

# removing temporary files

system("rm -f $TMP/genomic$$.fa $TMP/prot$$.fa $TMP/genomic_mask$$.fa $TMP/genewise$$ $TMP/genewise$$.gff");

exit(0);

##########################################################################
##                                                                      ##
##                              THE END                                 ##
##                                                                      ##
##########################################################################
##########################################################################


##########################################################################
##                                                                      ##
##                            SUBROUTINES                               ##
##                                                                      ##
##########################################################################


sub print_fasta {

    my ($entry, $file) = @_;
    my ($id, $seq) = split (/\n/,$entry);
    my ($i, $e) = (1, length($seq));
    print $file "$id\n";
    while ($i<=$e) {
	print  $file  substr($seq,$i-1,60)."\n"; 
	$i += 60;
    };
}

# This subrutine masks with N from 0 to pos1 and from pos2 to the end
sub print_fasta_masked {
   
    my ($entry, $file, $pos1, $pos2) = @_;
    my ($id, $seq) = split (/\n/,$entry);
    my ($i, $e) = (1, length($seq));
    if ($pos1 < 0){
	$pos1=0;
    }
    if ($pos2 <= $pos1){
	$pos2=$e;
    }      
    print $file "$id\n";
    while ($i<$pos1) {
        print $file "N";
        if (!($i % 60)){
	    print $file "\n";
	}
        $i++;
    }

    while ($i<=$e && $i<= $pos2) {
	print  $file  substr($seq,$i-1,1); 
        if (!($i % 60)){
	    print $file "\n";
	}
	$i++;
    };
    while ($i<$e) {
        print $file "N";
        if (!($i % 60)){
	    print $file "\n";
	}
        $i++;
    }
}
